'use strict';

var react = require('react');

function shallowEqual(objA, objB) {
  if (Object.is(objA, objB)) {
    return true;
  }

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  var keysA = Object.keys(objA);

  if (keysA.length !== Object.keys(objB).length) {
    return false;
  }

  for (var i = 0; i < keysA.length; i++) {
    if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !Object.is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

var reducer = function reducer(state, newState) {
  return newState;
};

function create(createState) {
  var listeners = new Set();

  var setState = function setState(partialState) {
    state = Object.assign({}, state, typeof partialState === 'function' ? partialState(state) : partialState);
    listeners.forEach(function (listener) {
      return listener(state);
    });
  };

  var getState = function getState() {
    return state;
  };

  var subscribe = function subscribe(listener) {
    listeners.add(listener);
    return function () {
      listeners["delete"](listener);
    };
  };

  var destroy = function destroy() {
    listeners.clear();
    state = {};
  };

  function useStore(selector, dependencies) {
    // State selector gets entire state if no selector was passed in
    var selectState = typeof selector === 'function' ? selector : getState;
    var selectStateRef = react.useRef(selectState);
    var dependenciesRef = react.useRef(dependencies);

    var _useReducer = react.useReducer(reducer, state, selectState),
        stateSlice = _useReducer[0],
        dispatch = _useReducer[1]; // Call new selector if no dependencies were passed in and selector has changed or dependencies were passed in and have changed


    if (!dependencies && selectState !== selectStateRef.current || !shallowEqual(dependencies, dependenciesRef.current)) stateSlice = selectState(state); // Store in ref to enable updating without rerunning subscribe/unsubscribe

    var stateSliceRef = react.useRef(stateSlice); // Update refs only after view has been updated

    react.useEffect(function () {
      return void (selectStateRef.current = selectState);
    }, [selectState]);
    react.useEffect(function () {
      return void (stateSliceRef.current = stateSlice);
    }, [stateSlice]); // Subscribe/unsubscribe to the store only on mount/unmount

    react.useEffect(function () {
      return subscribe(function () {
        // Use the last selector passed to useStore to get current state slice
        var selectedSlice = selectStateRef.current(state); // Shallow compare previous state slice with current and rerender only if changed

        if (!shallowEqual(stateSliceRef.current, selectedSlice)) dispatch(selectedSlice);
      });
    }, []);
    return stateSlice;
  }

  var state = createState(setState, getState);
  var api = {
    destroy: destroy,
    getState: getState,
    setState: setState,
    subscribe: subscribe
  };
  return [useStore, api];
}

module.exports = create;
